import React, { useMemo, useContext, forwardRef, useImperativeHandle, createContext, useState } from 'react'
import FormRender, { useForm } from 'form-render'
import { ConfigProvider } from 'antd'
import widgets from '../widgets'
import { isObject } from '../../utils'
import getWatch from './watch'

export const GlobalStore = createContext(null)

const XRender = (props, ref) => {

  useImperativeHandle(ref, () => (
    {
      getValues
    }
  ))

  const { schemaValues } = props
  const { schema, values } = schemaValues
  const { getPrefixCls } = useContext(ConfigProvider.ConfigContext)
  const prefixCls = getPrefixCls('pandaXform')
  const pandaXform = getPrefixCls()
  const [startTime, setStartTime] = useState(new Date().getTime())
  const form = useForm()

  const schemaForm = useMemo(() => {
    let schemaForm = schema ? (isObject(schema) ? schema : JSON.parse(schema)) : {}
    if (Array.isArray(values)) {
      let parent = schemaForm?.properties
      let parentObj = {}
      if (isObject(parent)) {
        for (let v in parent) {
          let child = parent[v]?.properties
          let childObj = {}
          if (isObject(child)) {
            for (let s in child) {
              let value = ''
              values.forEach(v => {
                if (v.fieldName === s) {
                  value = v.fieldValue
                }
              })
              childObj[s] = { ...child[s], presetValue: value || child[s].presetValue }
            }
          }
          if (JSON.stringify(childObj) !== '{}') {
            parentObj[v] = { ...parent[v], properties: childObj }
          }
        }
      }
      return { ...schema, properties: parentObj }
    }
    return schemaForm
  }, [schema, values])

  const watch = useMemo(() => {
    return getWatch(schema, form, startTime)
  }, [schemaForm, form, startTime])

  const getValues = async () => {
    let { data, errors } = await form.submit()
    let formValue = []
    Object.keys(data).map((k) => {
      if (isObject(data[k])) {
        Object.keys(data[k]).map((key) => {
          formValue.push({ fieldName: key, fieldValue: data[k][key] })
        })
      }
    })
    return {
      formValue,
      errors,
      relationForm: data.relationForm
    }
  }

  return (
    <div className={prefixCls}>
      <FormRender
        configProvider={{ prefixCls: pandaXform }}
        form={form}
        schema={schemaForm}
        mapping={{
          object: 'Header',
        }}
        widgets={widgets}
        watch={watch}
      />
    </div>
  )

}

export default forwardRef(XRender)
